#include <assert.h>

#include <aterm2.h>
#include <deprecated.h>
#include "Booleans.h"

/*{{{  conversion functions */

ATerm SDFstringToChars(const char *str)
{
  int len = strlen(str);
  int i;
  ATermList result = ATempty;

  for (i = len - 1; i >= 0; i--) {
    result = ATinsert(result, (ATerm) ATmakeInt(str[i]));
  }

  return (ATerm) result;
}

char *SDFcharsToString(ATerm arg)
{
  ATermList list = (ATermList) arg;
  int len = ATgetLength(list);
  int i;
  char *str;

  str = (char *) malloc(len+1);
  if (str == NULL) {
      return NULL;
  }

  for (i = 0; !ATisEmpty(list); list = ATgetNext(list), i++) {
    str[i] = (char) ATgetInt((ATermInt) ATgetFirst(list));
  }
  str[i] = '\0';

  return str;
}


/*}}}  */

/*{{{  typedefs */

typedef struct ATerm _SDFBool;
typedef struct ATerm _SDFBoolList;
typedef struct ATerm _SDFBoolElems;
typedef struct ATerm _SDFStart;
typedef struct ATerm _SDFOptLayout;

/*}}}  */

/*{{{  void SDFinitBooleansApi(void) */

void SDFinitBooleansApi(void)
{
  init_Booleans_dict();
}

/*}}}  */

/*{{{  protect functions */

void SDFprotectBool(SDFBool *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDFprotectBoolList(SDFBoolList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDFprotectBoolElems(SDFBoolElems *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDFprotectStart(SDFStart *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDFprotectOptLayout(SDFOptLayout *arg)
{
  ATprotect((ATerm*)((void*) arg));
}


/*}}}  */
/*{{{  term conversion functions */

/*{{{  SDFBool SDFBoolFromTerm(ATerm t) */

SDFBool SDFBoolFromTerm(ATerm t)
{
  return (SDFBool)t;
}

/*}}}  */
/*{{{  ATerm SDFBoolToTerm(SDFBool arg) */

ATerm SDFBoolToTerm(SDFBool arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDFBoolList SDFBoolListFromTerm(ATerm t) */

SDFBoolList SDFBoolListFromTerm(ATerm t)
{
  return (SDFBoolList)t;
}

/*}}}  */
/*{{{  ATerm SDFBoolListToTerm(SDFBoolList arg) */

ATerm SDFBoolListToTerm(SDFBoolList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDFBoolElems SDFBoolElemsFromTerm(ATerm t) */

SDFBoolElems SDFBoolElemsFromTerm(ATerm t)
{
  return (SDFBoolElems)t;
}

/*}}}  */
/*{{{  ATerm SDFBoolElemsToTerm(SDFBoolElems arg) */

ATerm SDFBoolElemsToTerm(SDFBoolElems arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDFStart SDFStartFromTerm(ATerm t) */

SDFStart SDFStartFromTerm(ATerm t)
{
  return (SDFStart)t;
}

/*}}}  */
/*{{{  ATerm SDFStartToTerm(SDFStart arg) */

ATerm SDFStartToTerm(SDFStart arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDFOptLayout SDFOptLayoutFromTerm(ATerm t) */

SDFOptLayout SDFOptLayoutFromTerm(ATerm t)
{
  return (SDFOptLayout)t;
}

/*}}}  */
/*{{{  ATerm SDFOptLayoutToTerm(SDFOptLayout arg) */

ATerm SDFOptLayoutToTerm(SDFOptLayout arg)
{
  return (ATerm)arg;
}

/*}}}  */

/*}}}  */
/*{{{  list functions */

int SDFgetBoolElemsLength (SDFBoolElems arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDFBoolElems SDFreverseBoolElems(SDFBoolElems arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDFBoolElems) result;
}
SDFBoolElems SDFappendBoolElems(SDFBoolElems arg0, SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBool arg1) {
  return SDFconcatBoolElems(arg0, wsAfterHead, wsAfterSep, SDFmakeBoolElemsSingle(arg1));
}
SDFBoolElems SDFconcatBoolElems(SDFBoolElems arg0, SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBoolElems arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDFmakeBoolElemsMany((SDFBool)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDFBoolElems) ATgetNext((ATermList) arg1);
  return (SDFBoolElems) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDFBoolElems SDFsliceBoolElems(SDFBoolElems arg, int start, int end) {
  return (SDFBoolElems) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDFBool SDFgetBoolElemsBoolAt(SDFBoolElems arg, int index) {
 return (SDFBool)ATelementAt((ATermList) arg,index * 4);
}
SDFBoolElems SDFreplaceBoolElemsBoolAt(SDFBoolElems arg, SDFBool elem, int index) {
 return (SDFBoolElems) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDFBoolElems SDFmakeBoolElems2(SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBool elem1, SDFBool elem2) {
  return SDFmakeBoolElemsMany(elem1, wsAfterHead, wsAfterSep, SDFmakeBoolElemsSingle(elem2));
}
SDFBoolElems SDFmakeBoolElems3(SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBool elem1, SDFBool elem2, SDFBool elem3) {
  return SDFmakeBoolElemsMany(elem1, wsAfterHead, wsAfterSep, SDFmakeBoolElems2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDFBoolElems SDFmakeBoolElems4(SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBool elem1, SDFBool elem2, SDFBool elem3, SDFBool elem4) {
  return SDFmakeBoolElemsMany(elem1, wsAfterHead, wsAfterSep, SDFmakeBoolElems3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDFBoolElems SDFmakeBoolElems5(SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBool elem1, SDFBool elem2, SDFBool elem3, SDFBool elem4, SDFBool elem5) {
  return SDFmakeBoolElemsMany(elem1, wsAfterHead, wsAfterSep, SDFmakeBoolElems4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDFBoolElems SDFmakeBoolElems6(SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBool elem1, SDFBool elem2, SDFBool elem3, SDFBool elem4, SDFBool elem5, SDFBool elem6) {
  return SDFmakeBoolElemsMany(elem1, wsAfterHead, wsAfterSep, SDFmakeBoolElems5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}

/*}}}  */
/*{{{  constructors */

/*{{{  SDFBool SDFmakeBoolTrue(void) */

SDFBool SDFmakeBoolTrue(void)
{
  return (SDFBool)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun3))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun6))), (ATerm)ATmakeAppl1(SDFafun7, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun8, (ATerm)ATmakeAppl1(SDFafun9, (ATerm)ATmakeAppl0(SDFafun3)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun3))));
}

/*}}}  */
/*{{{  SDFBool SDFmakeBoolFalse(void) */

SDFBool SDFmakeBoolFalse(void)
{
  return (SDFBool)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun10))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun6))), (ATerm)ATmakeAppl1(SDFafun7, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun8, (ATerm)ATmakeAppl1(SDFafun9, (ATerm)ATmakeAppl0(SDFafun10)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun10))));
}

/*}}}  */
/*{{{  SDFBool SDFmakeBoolAnd(SDFBool left, SDFOptLayout wsAfterLeft, SDFOptLayout wsAfterAmp, SDFBool right) */

SDFBool SDFmakeBoolAnd(SDFBool left, SDFOptLayout wsAfterLeft, SDFOptLayout wsAfterAmp, SDFBool right)
{
  return (SDFBool)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun6)))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun11, (ATerm)ATmakeAppl0(SDFafun12)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun13))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun11, (ATerm)ATmakeAppl0(SDFafun12)))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun6)))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun6))), (ATerm)ATmakeAppl1(SDFafun7, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeAppl0(SDFafun15))), (ATerm)ATmakeAppl1(SDFafun8, (ATerm)ATmakeAppl1(SDFafun9, (ATerm)ATmakeAppl0(SDFafun16)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterAmp), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun13))), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDFBool SDFmakeBoolOr(SDFBool left, SDFOptLayout wsAfterLeft, SDFOptLayout wsAfterBar, SDFBool right) */

SDFBool SDFmakeBoolOr(SDFBool left, SDFOptLayout wsAfterLeft, SDFOptLayout wsAfterBar, SDFBool right)
{
  return (SDFBool)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun6)))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun11, (ATerm)ATmakeAppl0(SDFafun12)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun17))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun11, (ATerm)ATmakeAppl0(SDFafun12)))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun6)))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun6))), (ATerm)ATmakeAppl1(SDFafun7, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun14, (ATerm)ATmakeAppl0(SDFafun15))), (ATerm)ATmakeAppl1(SDFafun8, (ATerm)ATmakeAppl1(SDFafun9, (ATerm)ATmakeAppl0(SDFafun18)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterBar), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun17))), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDFBoolList SDFmakeBoolListElems(SDFOptLayout wsAfterBracketOpen, SDFBoolElems elems, SDFOptLayout wsAfterElems) */

SDFBoolList SDFmakeBoolListElems(SDFOptLayout wsAfterBracketOpen, SDFBoolElems elems, SDFOptLayout wsAfterElems)
{
  return (SDFBoolList)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun19))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun11, (ATerm)ATmakeAppl0(SDFafun12)))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl2(SDFafun20, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun6)), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun21))))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun11, (ATerm)ATmakeAppl0(SDFafun12)))), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun22))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun23))), (ATerm)ATmakeAppl1(SDFafun7, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun8, (ATerm)ATmakeAppl1(SDFafun9, (ATerm)ATmakeAppl0(SDFafun24)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun19))), (ATerm) wsAfterElems), (ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl1(SDFafun25, (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl2(SDFafun20, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun6)), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun21))))), (ATerm) elems)), (ATerm) wsAfterBracketOpen), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun22))));
}

/*}}}  */
/*{{{  SDFBoolElems SDFmakeBoolElemsEmpty(void) */

SDFBoolElems SDFmakeBoolElemsEmpty(void)
{
  return (SDFBoolElems)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDFBoolElems SDFmakeBoolElemsSingle(SDFBool head) */

SDFBoolElems SDFmakeBoolElemsSingle(SDFBool head)
{
  return (SDFBoolElems)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDFBoolElems SDFmakeBoolElemsMany(SDFBool head, SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBoolElems tail) */

SDFBoolElems SDFmakeBoolElemsMany(SDFBool head, SDFOptLayout wsAfterHead, SDFOptLayout wsAfterSep, SDFBoolElems tail)
{
  if (SDFisBoolElemsEmpty(tail)) {
    return SDFmakeBoolElemsSingle(head);
  }
  return (SDFBoolElems)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl1(SDFafun2, (ATerm)ATmakeAppl0(SDFafun21))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDFStart SDFmakeStartBoolList(SDFOptLayout wsBefore, SDFBoolList topBoolList, SDFOptLayout wsAfter, int ambCnt) */

SDFStart SDFmakeStartBoolList(SDFOptLayout wsBefore, SDFBoolList topBoolList, SDFOptLayout wsAfter, int ambCnt)
{
  return (SDFStart)(ATerm)ATmakeAppl2(SDFafun26, (ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun11, (ATerm)ATmakeAppl0(SDFafun12)))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun23)))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun11, (ATerm)ATmakeAppl0(SDFafun12)))), (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun27)), (ATerm)ATmakeAppl0(SDFafun28)), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) wsAfter), (ATerm) topBoolList), (ATerm) wsBefore)), (ATerm) (ATerm) ATmakeInt(ambCnt));
}

/*}}}  */
/*{{{  SDFStart SDFmakeStartBool(SDFOptLayout wsBefore, SDFBool topBool, SDFOptLayout wsAfter, int ambCnt) */

SDFStart SDFmakeStartBool(SDFOptLayout wsBefore, SDFBool topBool, SDFOptLayout wsAfter, int ambCnt)
{
  return (SDFStart)(ATerm)ATmakeAppl2(SDFafun26, (ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun11, (ATerm)ATmakeAppl0(SDFafun12)))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun6)))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun11, (ATerm)ATmakeAppl0(SDFafun12)))), (ATerm)ATmakeAppl1(SDFafun5, (ATerm)ATmakeAppl0(SDFafun27)), (ATerm)ATmakeAppl0(SDFafun28)), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) wsAfter), (ATerm) topBool), (ATerm) wsBefore)), (ATerm) (ATerm) ATmakeInt(ambCnt));
}

/*}}}  */
/*{{{  SDFOptLayout SDFmakeOptLayoutAbsent(void) */

SDFOptLayout SDFmakeOptLayoutAbsent(void)
{
  return (SDFOptLayout)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATempty, (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun11, (ATerm)ATmakeAppl0(SDFafun12))), (ATerm)ATmakeAppl0(SDFafun28)), (ATerm)ATempty);
}

/*}}}  */
/*{{{  SDFOptLayout SDFmakeOptLayoutPresent(const char* string) */

SDFOptLayout SDFmakeOptLayoutPresent(const char* string)
{
  return (SDFOptLayout)(ATerm)ATmakeAppl2(SDFafun0, (ATerm)ATmakeAppl3(SDFafun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl0(SDFafun12))), (ATerm)ATmakeAppl1(SDFafun4, (ATerm)ATmakeAppl1(SDFafun11, (ATerm)ATmakeAppl0(SDFafun12))), (ATerm)ATmakeAppl0(SDFafun28)), (ATerm) ((ATerm) SDFstringToChars(string)));
}

/*}}}  */

/*}}}  */
/*{{{  equality functions */

ATbool SDFisEqualBool(SDFBool arg0, SDFBool arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDFisEqualBoolList(SDFBoolList arg0, SDFBoolList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDFisEqualBoolElems(SDFBoolElems arg0, SDFBoolElems arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDFisEqualStart(SDFStart arg0, SDFStart arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDFisEqualOptLayout(SDFOptLayout arg0, SDFOptLayout arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

/*}}}  */
/*{{{  SDFBool accessors */

/*{{{  ATbool SDFisValidBool(SDFBool arg) */

ATbool SDFisValidBool(SDFBool arg)
{
  if (SDFisBoolTrue(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolFalse(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolAnd(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolOr(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolTrue(SDFBool arg) */

inline ATbool SDFisBoolTrue(SDFBool arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternBoolTrue);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolFalse(SDFBool arg) */

inline ATbool SDFisBoolFalse(SDFBool arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternBoolFalse);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolAnd(SDFBool arg) */

inline ATbool SDFisBoolAnd(SDFBool arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternBoolAnd, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolOr(SDFBool arg) */

inline ATbool SDFisBoolOr(SDFBool arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternBoolOr, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDFhasBoolLeft(SDFBool arg) */

ATbool SDFhasBoolLeft(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolOr(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolWsAfterLeft(SDFBool arg) */

ATbool SDFhasBoolWsAfterLeft(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolOr(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolWsAfterAmp(SDFBool arg) */

ATbool SDFhasBoolWsAfterAmp(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolRight(SDFBool arg) */

ATbool SDFhasBoolRight(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolOr(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolWsAfterBar(SDFBool arg) */

ATbool SDFhasBoolWsAfterBar(SDFBool arg)
{
  if (SDFisBoolOr(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDFBool SDFgetBoolLeft(SDFBool arg) */

SDFBool SDFgetBoolLeft(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFBool)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (SDFBool)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolWsAfterLeft(SDFBool arg) */

SDFOptLayout SDFgetBoolWsAfterLeft(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else 
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolWsAfterAmp(SDFBool arg) */

SDFOptLayout SDFgetBoolWsAfterAmp(SDFBool arg)
{
  
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDFBool SDFgetBoolRight(SDFBool arg) */

SDFBool SDFgetBoolRight(SDFBool arg)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFBool)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else 
    return (SDFBool)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolWsAfterBar(SDFBool arg) */

SDFOptLayout SDFgetBoolWsAfterBar(SDFBool arg)
{
  
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDFBool SDFsetBoolLeft(SDFBool arg, SDFBool left) */

SDFBool SDFsetBoolLeft(SDFBool arg, SDFBool left)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }
  else if (SDFisBoolOr(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }

  ATabort("Bool has no Left: %t\n", arg);
  return (SDFBool)NULL;
}

/*}}}  */
/*{{{  SDFBool SDFsetBoolWsAfterLeft(SDFBool arg, SDFOptLayout wsAfterLeft) */

SDFBool SDFsetBoolWsAfterLeft(SDFBool arg, SDFOptLayout wsAfterLeft)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }
  else if (SDFisBoolOr(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }

  ATabort("Bool has no WsAfterLeft: %t\n", arg);
  return (SDFBool)NULL;
}

/*}}}  */
/*{{{  SDFBool SDFsetBoolWsAfterAmp(SDFBool arg, SDFOptLayout wsAfterAmp) */

SDFBool SDFsetBoolWsAfterAmp(SDFBool arg, SDFOptLayout wsAfterAmp)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterAmp), 3), 1);
  }

  ATabort("Bool has no WsAfterAmp: %t\n", arg);
  return (SDFBool)NULL;
}

/*}}}  */
/*{{{  SDFBool SDFsetBoolRight(SDFBool arg, SDFBool right) */

SDFBool SDFsetBoolRight(SDFBool arg, SDFBool right)
{
  if (SDFisBoolAnd(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 4), 1);
  }
  else if (SDFisBoolOr(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 4), 1);
  }

  ATabort("Bool has no Right: %t\n", arg);
  return (SDFBool)NULL;
}

/*}}}  */
/*{{{  SDFBool SDFsetBoolWsAfterBar(SDFBool arg, SDFOptLayout wsAfterBar) */

SDFBool SDFsetBoolWsAfterBar(SDFBool arg, SDFOptLayout wsAfterBar)
{
  if (SDFisBoolOr(arg)) {
    return (SDFBool)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBar), 3), 1);
  }

  ATabort("Bool has no WsAfterBar: %t\n", arg);
  return (SDFBool)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDFBoolList accessors */

/*{{{  ATbool SDFisValidBoolList(SDFBoolList arg) */

ATbool SDFisValidBoolList(SDFBoolList arg)
{
  if (SDFisBoolListElems(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolListElems(SDFBoolList arg) */

inline ATbool SDFisBoolListElems(SDFBoolList arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDFpatternBoolListElems, NULL, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolListWsAfterBracketOpen(SDFBoolList arg) */

ATbool SDFhasBoolListWsAfterBracketOpen(SDFBoolList arg)
{
  if (SDFisBoolListElems(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolListElems(SDFBoolList arg) */

ATbool SDFhasBoolListElems(SDFBoolList arg)
{
  if (SDFisBoolListElems(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolListWsAfterElems(SDFBoolList arg) */

ATbool SDFhasBoolListWsAfterElems(SDFBoolList arg)
{
  if (SDFisBoolListElems(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolListWsAfterBracketOpen(SDFBoolList arg) */

SDFOptLayout SDFgetBoolListWsAfterBracketOpen(SDFBoolList arg)
{
  
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDFBoolElems SDFgetBoolListElems(SDFBoolList arg) */

SDFBoolElems SDFgetBoolListElems(SDFBoolList arg)
{
  
    return (SDFBoolElems)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), 1);
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolListWsAfterElems(SDFBoolList arg) */

SDFOptLayout SDFgetBoolListWsAfterElems(SDFBoolList arg)
{
  
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDFBoolList SDFsetBoolListWsAfterBracketOpen(SDFBoolList arg, SDFOptLayout wsAfterBracketOpen) */

SDFBoolList SDFsetBoolListWsAfterBracketOpen(SDFBoolList arg, SDFOptLayout wsAfterBracketOpen)
{
  if (SDFisBoolListElems(arg)) {
    return (SDFBoolList)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBracketOpen), 1), 1);
  }

  ATabort("BoolList has no WsAfterBracketOpen: %t\n", arg);
  return (SDFBoolList)NULL;
}

/*}}}  */
/*{{{  SDFBoolList SDFsetBoolListElems(SDFBoolList arg, SDFBoolElems elems) */

SDFBoolList SDFsetBoolListElems(SDFBoolList arg, SDFBoolElems elems)
{
  if (SDFisBoolListElems(arg)) {
    return (SDFBoolList)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), (ATerm)((ATerm) elems), 1), 2), 1);
  }

  ATabort("BoolList has no Elems: %t\n", arg);
  return (SDFBoolList)NULL;
}

/*}}}  */
/*{{{  SDFBoolList SDFsetBoolListWsAfterElems(SDFBoolList arg, SDFOptLayout wsAfterElems) */

SDFBoolList SDFsetBoolListWsAfterElems(SDFBoolList arg, SDFOptLayout wsAfterElems)
{
  if (SDFisBoolListElems(arg)) {
    return (SDFBoolList)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterElems), 3), 1);
  }

  ATabort("BoolList has no WsAfterElems: %t\n", arg);
  return (SDFBoolList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDFBoolElems accessors */

/*{{{  ATbool SDFisValidBoolElems(SDFBoolElems arg) */

ATbool SDFisValidBoolElems(SDFBoolElems arg)
{
  if (SDFisBoolElemsEmpty(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolElemsSingle(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolElemsEmpty(SDFBoolElems arg) */

inline ATbool SDFisBoolElemsEmpty(SDFBoolElems arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDFpatternBoolElemsEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolElemsSingle(SDFBoolElems arg) */

inline ATbool SDFisBoolElemsSingle(SDFBoolElems arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternBoolElemsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisBoolElemsMany(SDFBoolElems arg) */

inline ATbool SDFisBoolElemsMany(SDFBoolElems arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternBoolElemsMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDFhasBoolElemsHead(SDFBoolElems arg) */

ATbool SDFhasBoolElemsHead(SDFBoolElems arg)
{
  if (SDFisBoolElemsSingle(arg)) {
    return ATtrue;
  }
  else if (SDFisBoolElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolElemsWsAfterHead(SDFBoolElems arg) */

ATbool SDFhasBoolElemsWsAfterHead(SDFBoolElems arg)
{
  if (SDFisBoolElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolElemsWsAfterSep(SDFBoolElems arg) */

ATbool SDFhasBoolElemsWsAfterSep(SDFBoolElems arg)
{
  if (SDFisBoolElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasBoolElemsTail(SDFBoolElems arg) */

ATbool SDFhasBoolElemsTail(SDFBoolElems arg)
{
  if (SDFisBoolElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDFBoolElems SDFgetBoolElemsTail(SDFBoolElems arg) */

SDFBoolElems SDFgetBoolElemsTail(SDFBoolElems arg)
{
  assert(!SDFisBoolElemsEmpty(arg) && "getTail on an empty list");
  if (SDFisBoolElemsSingle(arg)) {
    return (SDFBoolElems) SDFmakeBoolElemsEmpty();
  }
  else {
  
    return (SDFBoolElems)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDFBool SDFgetBoolElemsHead(SDFBoolElems arg) */

SDFBool SDFgetBoolElemsHead(SDFBoolElems arg)
{
  if (SDFisBoolElemsSingle(arg)) {
    return (SDFBool)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDFBool)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolElemsWsAfterHead(SDFBoolElems arg) */

SDFOptLayout SDFgetBoolElemsWsAfterHead(SDFBoolElems arg)
{
  
    return (SDFOptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetBoolElemsWsAfterSep(SDFBoolElems arg) */

SDFOptLayout SDFgetBoolElemsWsAfterSep(SDFBoolElems arg)
{
  
    return (SDFOptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDFBoolElems SDFsetBoolElemsHead(SDFBoolElems arg, SDFBool head) */

SDFBoolElems SDFsetBoolElemsHead(SDFBoolElems arg, SDFBool head)
{
  if (SDFisBoolElemsSingle(arg)) {
    return (SDFBoolElems)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDFisBoolElemsMany(arg)) {
    return (SDFBoolElems)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("BoolElems has no Head: %t\n", arg);
  return (SDFBoolElems)NULL;
}

/*}}}  */
/*{{{  SDFBoolElems SDFsetBoolElemsWsAfterHead(SDFBoolElems arg, SDFOptLayout wsAfterHead) */

SDFBoolElems SDFsetBoolElemsWsAfterHead(SDFBoolElems arg, SDFOptLayout wsAfterHead)
{
  if (SDFisBoolElemsMany(arg)) {
    return (SDFBoolElems)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("BoolElems has no WsAfterHead: %t\n", arg);
  return (SDFBoolElems)NULL;
}

/*}}}  */
/*{{{  SDFBoolElems SDFsetBoolElemsWsAfterSep(SDFBoolElems arg, SDFOptLayout wsAfterSep) */

SDFBoolElems SDFsetBoolElemsWsAfterSep(SDFBoolElems arg, SDFOptLayout wsAfterSep)
{
  if (SDFisBoolElemsMany(arg)) {
    return (SDFBoolElems)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("BoolElems has no WsAfterSep: %t\n", arg);
  return (SDFBoolElems)NULL;
}

/*}}}  */
/*{{{  SDFBoolElems SDFsetBoolElemsTail(SDFBoolElems arg, SDFBoolElems tail) */

SDFBoolElems SDFsetBoolElemsTail(SDFBoolElems arg, SDFBoolElems tail)
{
  if (SDFisBoolElemsMany(arg)) {
    return (SDFBoolElems)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("BoolElems has no Tail: %t\n", arg);
  return (SDFBoolElems)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDFStart accessors */

/*{{{  ATbool SDFisValidStart(SDFStart arg) */

ATbool SDFisValidStart(SDFStart arg)
{
  if (SDFisStartBoolList(arg)) {
    return ATtrue;
  }
  else if (SDFisStartBool(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisStartBoolList(SDFStart arg) */

inline ATbool SDFisStartBoolList(SDFStart arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternStartBoolList, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisStartBool(SDFStart arg) */

inline ATbool SDFisStartBool(SDFStart arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternStartBool, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDFhasStartWsBefore(SDFStart arg) */

ATbool SDFhasStartWsBefore(SDFStart arg)
{
  if (SDFisStartBoolList(arg)) {
    return ATtrue;
  }
  else if (SDFisStartBool(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasStartTopBoolList(SDFStart arg) */

ATbool SDFhasStartTopBoolList(SDFStart arg)
{
  if (SDFisStartBoolList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasStartWsAfter(SDFStart arg) */

ATbool SDFhasStartWsAfter(SDFStart arg)
{
  if (SDFisStartBoolList(arg)) {
    return ATtrue;
  }
  else if (SDFisStartBool(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasStartAmbCnt(SDFStart arg) */

ATbool SDFhasStartAmbCnt(SDFStart arg)
{
  if (SDFisStartBoolList(arg)) {
    return ATtrue;
  }
  else if (SDFisStartBool(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDFhasStartTopBool(SDFStart arg) */

ATbool SDFhasStartTopBool(SDFStart arg)
{
  if (SDFisStartBool(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetStartWsBefore(SDFStart arg) */

SDFOptLayout SDFgetStartWsBefore(SDFStart arg)
{
  if (SDFisStartBoolList(arg)) {
    return (SDFOptLayout)ATgetFirst((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1));
  }
  else 
    return (SDFOptLayout)ATgetFirst((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1));
}

/*}}}  */
/*{{{  SDFBoolList SDFgetStartTopBoolList(SDFStart arg) */

SDFBoolList SDFgetStartTopBoolList(SDFStart arg)
{
  
    return (SDFBoolList)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 1);
}

/*}}}  */
/*{{{  SDFOptLayout SDFgetStartWsAfter(SDFStart arg) */

SDFOptLayout SDFgetStartWsAfter(SDFStart arg)
{
  if (SDFisStartBoolList(arg)) {
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 2);
  }
  else 
    return (SDFOptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 2);
}

/*}}}  */
/*{{{  int SDFgetStartAmbCnt(SDFStart arg) */

int SDFgetStartAmbCnt(SDFStart arg)
{
  if (SDFisStartBoolList(arg)) {
    return (int)ATgetInt((ATermInt) ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (int)ATgetInt((ATermInt) ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDFBool SDFgetStartTopBool(SDFStart arg) */

SDFBool SDFgetStartTopBool(SDFStart arg)
{
  
    return (SDFBool)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 1);
}

/*}}}  */
/*{{{  SDFStart SDFsetStartWsBefore(SDFStart arg, SDFOptLayout wsBefore) */

SDFStart SDFsetStartWsBefore(SDFStart arg, SDFOptLayout wsBefore)
{
  if (SDFisStartBoolList(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsBefore), 0), 1), 0);
  }
  else if (SDFisStartBool(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsBefore), 0), 1), 0);
  }

  ATabort("Start has no WsBefore: %t\n", arg);
  return (SDFStart)NULL;
}

/*}}}  */
/*{{{  SDFStart SDFsetStartTopBoolList(SDFStart arg, SDFBoolList topBoolList) */

SDFStart SDFsetStartTopBoolList(SDFStart arg, SDFBoolList topBoolList)
{
  if (SDFisStartBoolList(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) topBoolList), 1), 1), 0);
  }

  ATabort("Start has no TopBoolList: %t\n", arg);
  return (SDFStart)NULL;
}

/*}}}  */
/*{{{  SDFStart SDFsetStartWsAfter(SDFStart arg, SDFOptLayout wsAfter) */

SDFStart SDFsetStartWsAfter(SDFStart arg, SDFOptLayout wsAfter)
{
  if (SDFisStartBoolList(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsAfter), 2), 1), 0);
  }
  else if (SDFisStartBool(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsAfter), 2), 1), 0);
  }

  ATabort("Start has no WsAfter: %t\n", arg);
  return (SDFStart)NULL;
}

/*}}}  */
/*{{{  SDFStart SDFsetStartAmbCnt(SDFStart arg, int ambCnt) */

SDFStart SDFsetStartAmbCnt(SDFStart arg, int ambCnt)
{
  if (SDFisStartBoolList(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(ambCnt)), 1);
  }
  else if (SDFisStartBool(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(ambCnt)), 1);
  }

  ATabort("Start has no AmbCnt: %t\n", arg);
  return (SDFStart)NULL;
}

/*}}}  */
/*{{{  SDFStart SDFsetStartTopBool(SDFStart arg, SDFBool topBool) */

SDFStart SDFsetStartTopBool(SDFStart arg, SDFBool topBool)
{
  if (SDFisStartBool(arg)) {
    return (SDFStart)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) topBool), 1), 1), 0);
  }

  ATabort("Start has no TopBool: %t\n", arg);
  return (SDFStart)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDFOptLayout accessors */

/*{{{  ATbool SDFisValidOptLayout(SDFOptLayout arg) */

ATbool SDFisValidOptLayout(SDFOptLayout arg)
{
  if (SDFisOptLayoutAbsent(arg)) {
    return ATtrue;
  }
  else if (SDFisOptLayoutPresent(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDFisOptLayoutAbsent(SDFOptLayout arg) */

inline ATbool SDFisOptLayoutAbsent(SDFOptLayout arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternOptLayoutAbsent);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDFisOptLayoutPresent(SDFOptLayout arg) */

inline ATbool SDFisOptLayoutPresent(SDFOptLayout arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDFpatternOptLayoutPresent, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDFhasOptLayoutString(SDFOptLayout arg) */

ATbool SDFhasOptLayoutString(SDFOptLayout arg)
{
  if (SDFisOptLayoutPresent(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char* SDFgetOptLayoutString(SDFOptLayout arg) */

char* SDFgetOptLayoutString(SDFOptLayout arg)
{
  
    return (char*)SDFcharsToString((ATerm)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDFOptLayout SDFsetOptLayoutString(SDFOptLayout arg, const char* string) */

SDFOptLayout SDFsetOptLayoutString(SDFOptLayout arg, const char* string)
{
  if (SDFisOptLayoutPresent(arg)) {
    return (SDFOptLayout)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) ((ATerm) SDFstringToChars(string))), 1);
  }

  ATabort("OptLayout has no String: %t\n", arg);
  return (SDFOptLayout)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  sort visitors */

/*{{{  SDFBool SDFvisitBool(SDFBool arg, SDFOptLayout (*acceptWsAfterLeft)(SDFOptLayout), SDFOptLayout (*acceptWsAfterAmp)(SDFOptLayout), SDFOptLayout (*acceptWsAfterBar)(SDFOptLayout)) */

SDFBool SDFvisitBool(SDFBool arg, SDFOptLayout (*acceptWsAfterLeft)(SDFOptLayout), SDFOptLayout (*acceptWsAfterAmp)(SDFOptLayout), SDFOptLayout (*acceptWsAfterBar)(SDFOptLayout))
{
  if (SDFisBoolTrue(arg)) {
    return SDFmakeBoolTrue();
  }
  if (SDFisBoolFalse(arg)) {
    return SDFmakeBoolFalse();
  }
  if (SDFisBoolAnd(arg)) {
    return SDFmakeBoolAnd(
        SDFvisitBool(SDFgetBoolLeft(arg), acceptWsAfterLeft, acceptWsAfterAmp, acceptWsAfterBar),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDFgetBoolWsAfterLeft(arg)) : SDFgetBoolWsAfterLeft(arg),
        acceptWsAfterAmp ? acceptWsAfterAmp(SDFgetBoolWsAfterAmp(arg)) : SDFgetBoolWsAfterAmp(arg),
        SDFvisitBool(SDFgetBoolRight(arg), acceptWsAfterLeft, acceptWsAfterAmp, acceptWsAfterBar));
  }
  if (SDFisBoolOr(arg)) {
    return SDFmakeBoolOr(
        SDFvisitBool(SDFgetBoolLeft(arg), acceptWsAfterLeft, acceptWsAfterAmp, acceptWsAfterBar),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDFgetBoolWsAfterLeft(arg)) : SDFgetBoolWsAfterLeft(arg),
        acceptWsAfterBar ? acceptWsAfterBar(SDFgetBoolWsAfterBar(arg)) : SDFgetBoolWsAfterBar(arg),
        SDFvisitBool(SDFgetBoolRight(arg), acceptWsAfterLeft, acceptWsAfterAmp, acceptWsAfterBar));
  }
  ATabort("not a Bool: %t\n", arg);
  return (SDFBool)NULL;
}

/*}}}  */
/*{{{  SDFBoolList SDFvisitBoolList(SDFBoolList arg, SDFOptLayout (*acceptWsAfterBracketOpen)(SDFOptLayout), SDFBoolElems (*acceptElems)(SDFBoolElems), SDFOptLayout (*acceptWsAfterElems)(SDFOptLayout)) */

SDFBoolList SDFvisitBoolList(SDFBoolList arg, SDFOptLayout (*acceptWsAfterBracketOpen)(SDFOptLayout), SDFBoolElems (*acceptElems)(SDFBoolElems), SDFOptLayout (*acceptWsAfterElems)(SDFOptLayout))
{
  if (SDFisBoolListElems(arg)) {
    return SDFmakeBoolListElems(
        acceptWsAfterBracketOpen ? acceptWsAfterBracketOpen(SDFgetBoolListWsAfterBracketOpen(arg)) : SDFgetBoolListWsAfterBracketOpen(arg),
        acceptElems ? acceptElems(SDFgetBoolListElems(arg)) : SDFgetBoolListElems(arg),
        acceptWsAfterElems ? acceptWsAfterElems(SDFgetBoolListWsAfterElems(arg)) : SDFgetBoolListWsAfterElems(arg));
  }
  ATabort("not a BoolList: %t\n", arg);
  return (SDFBoolList)NULL;
}

/*}}}  */
/*{{{  SDFBoolElems SDFvisitBoolElems(SDFBoolElems arg, SDFBool (*acceptHead)(SDFBool), SDFOptLayout (*acceptWsAfterHead)(SDFOptLayout), SDFOptLayout (*acceptWsAfterSep)(SDFOptLayout)) */

SDFBoolElems SDFvisitBoolElems(SDFBoolElems arg, SDFBool (*acceptHead)(SDFBool), SDFOptLayout (*acceptWsAfterHead)(SDFOptLayout), SDFOptLayout (*acceptWsAfterSep)(SDFOptLayout))
{
  if (SDFisBoolElemsEmpty(arg)) {
    return SDFmakeBoolElemsEmpty();
  }
  if (SDFisBoolElemsSingle(arg)) {
    return SDFmakeBoolElemsSingle(
        acceptHead ? acceptHead(SDFgetBoolElemsHead(arg)) : SDFgetBoolElemsHead(arg));
  }
  if (SDFisBoolElemsMany(arg)) {
    return SDFmakeBoolElemsMany(
        acceptHead ? acceptHead(SDFgetBoolElemsHead(arg)) : SDFgetBoolElemsHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDFgetBoolElemsWsAfterHead(arg)) : SDFgetBoolElemsWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDFgetBoolElemsWsAfterSep(arg)) : SDFgetBoolElemsWsAfterSep(arg),
        SDFvisitBoolElems(SDFgetBoolElemsTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a BoolElems: %t\n", arg);
  return (SDFBoolElems)NULL;
}

/*}}}  */
/*{{{  SDFStart SDFvisitStart(SDFStart arg, SDFOptLayout (*acceptWsBefore)(SDFOptLayout), SDFBoolList (*acceptTopBoolList)(SDFBoolList), SDFOptLayout (*acceptWsAfter)(SDFOptLayout), int (*acceptAmbCnt)(int), SDFBool (*acceptTopBool)(SDFBool)) */

SDFStart SDFvisitStart(SDFStart arg, SDFOptLayout (*acceptWsBefore)(SDFOptLayout), SDFBoolList (*acceptTopBoolList)(SDFBoolList), SDFOptLayout (*acceptWsAfter)(SDFOptLayout), int (*acceptAmbCnt)(int), SDFBool (*acceptTopBool)(SDFBool))
{
  if (SDFisStartBoolList(arg)) {
    return SDFmakeStartBoolList(
        acceptWsBefore ? acceptWsBefore(SDFgetStartWsBefore(arg)) : SDFgetStartWsBefore(arg),
        acceptTopBoolList ? acceptTopBoolList(SDFgetStartTopBoolList(arg)) : SDFgetStartTopBoolList(arg),
        acceptWsAfter ? acceptWsAfter(SDFgetStartWsAfter(arg)) : SDFgetStartWsAfter(arg),
        acceptAmbCnt ? acceptAmbCnt(SDFgetStartAmbCnt(arg)) : SDFgetStartAmbCnt(arg));
  }
  if (SDFisStartBool(arg)) {
    return SDFmakeStartBool(
        acceptWsBefore ? acceptWsBefore(SDFgetStartWsBefore(arg)) : SDFgetStartWsBefore(arg),
        acceptTopBool ? acceptTopBool(SDFgetStartTopBool(arg)) : SDFgetStartTopBool(arg),
        acceptWsAfter ? acceptWsAfter(SDFgetStartWsAfter(arg)) : SDFgetStartWsAfter(arg),
        acceptAmbCnt ? acceptAmbCnt(SDFgetStartAmbCnt(arg)) : SDFgetStartAmbCnt(arg));
  }
  ATabort("not a Start: %t\n", arg);
  return (SDFStart)NULL;
}

/*}}}  */
/*{{{  SDFOptLayout SDFvisitOptLayout(SDFOptLayout arg, char* (*acceptString)(char*)) */

SDFOptLayout SDFvisitOptLayout(SDFOptLayout arg, char* (*acceptString)(char*))
{
  if (SDFisOptLayoutAbsent(arg)) {
    return SDFmakeOptLayoutAbsent();
  }
  if (SDFisOptLayoutPresent(arg)) {
    return SDFmakeOptLayoutPresent(
        acceptString ? acceptString(SDFgetOptLayoutString(arg)) : SDFgetOptLayoutString(arg));
  }
  ATabort("not a OptLayout: %t\n", arg);
  return (SDFOptLayout)NULL;
}

/*}}}  */

/*}}}  */
